@@ -1,4 +1,4 @@
-# 0.0.40
+# 0.1.2a
 ##################################################################################
 #   MIT License
 #
@@ -29,7 +29,8 @@ import logging
 import os
 import signal
 import sys
-from concurrent.futures import ThreadPoolExecutor
+from concurrent.futures import ThreadPoolExecutor, wait
+from concurrent.futures._base import ALL_COMPLETED, Executor, as_completed
 from time import sleep, strftime, time
 import urllib3
 ##############
@@ -63,7 +64,6 @@ username = getpass.getuser()
 
 exit_app = False
 urllib3.disable_warnings()
-
 # print(sys.version)
 
 
@@ -115,9 +115,7 @@ class GBase:
     sleeper = 2
     pool = ThreadPoolExecutor(200)
     radio_base_dir = os.path.dirname(os.path.abspath(__file__)) + '//radiostations'  # if not set set_radio_base_dir()
-    # radio_base_dir = ghetto_folder
     settings_path = os.path.dirname(os.path.abspath(__file__)) + '//settings.ini'  # if not set in set_settings_path()
-    # settings_path = radio_base_dir + '//settings.ini'
     path = os.getcwd()
     path_to = path + '//'
     timer = 0
@@ -128,18 +126,6 @@ class GBase:
         self.radio_base_dir = radio_base_dir
         self.settings_path = settings_path
 
-    def set_settings_path(self, settings_path):
-        self.settings_path = settings_path
-
-    def get_settings_path(self):
-        return self.settings_path
-
-    def set_radio_base_dir(self, radio_base_dir):
-        self.radio_base_dir = radio_base_dir
-
-    def get_radio_base_dir(self):
-        return self.radio_base_dir
-
     @staticmethod
     def make_directory(str_path):
         access_rights = 0o755
@@ -147,7 +133,6 @@ class GBase:
             os.mkdir(str_path, access_rights)
         except FileExistsError:
             pass
-            #  print(repr(ex))
             # print(' Folder exists: ' + str_path)
             return False
         else:
@@ -181,13 +166,14 @@ class GBase:
 
 
 class GIni(GBase):
-    ini_keys = {}  # cls attribute to store selections from ini file, works because of key[key] = value, else not
+
+    ini_keys = {}        # cls attribute to store selections from ini file, works because of key[key] = value, else not
     srv_param_dict = {}  # all ini keys plus short url, suffix, server type stuff
-    song_dict = {}  # recorder reads song title from key, related to the url in settings.ini
+    song_dict = {}       # recorder reads song title from key, related to the url in settings.ini
     start_stop_recording = {}  # ini key: 'start' , 'stop'; while loop check start, working check stop go upper while
     # ini_key + '_single_title', ini_key + '_rec_from_here'
-    cost_current_ini = ''  # ini key for cost_dict calc
-    cost_dict = {}  # stores len of received headers to calc amount of data searching strings per day
+    cost_current_ini = ''  # ini key for cost_dict calc / should be a dict
+    cost_dict = {}        # stores len of received headers to calc amount of data searching strings per day
     fail_meta_dict = {}  # can not read metadata from stream, no data
     # list of search strings delimiter blank, first key is named 'STRINGS': Britney Phantom ไม่เคยจะจำ Elton Jim techno
     search_dict = {'STRINGS': 'Britney Spears ไม่เคยจะจำ Elton AC/DC techno Band feat. mix'}  # only show it is working
@@ -270,29 +256,22 @@ class GIni(GBase):
 
 
 class GNet(GBase):
-    is_shoutcast_server = False
-    is_icecast_server = False
-    is_icecast_json = False
-    is_unknown_server = False
-    is_no_meta_data_avail = False
-
+    
+    is_shout_dict = {}
+    is_ice_dict = {}
+    is_ice_json_dict = {}
+    is_unk_dict = {}
+    is_no_meta_avail_dict = {}
+    
     http_pool = urllib3.PoolManager(num_pools=200)
     request = ''
     ua_standard = {'Icy-MetaData': '1', 'user-agent': "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 "
-                                                      "(KHTML, like Gecko) Chrome/52.0.2743.82 Safari/537.36"}
+                      "(KHTML, like Gecko) Chrome/52.0.2743.82 Safari/537.36"}
     query_shoutcast = '/currentsong'
 
     def __init__(self):
         super().__init__()
 
-    @staticmethod
-    def reset_switch():
-        GNet.is_shoutcast_server = False
-        GNet.is_icecast_server = False
-        GNet.is_icecast_json = False
-        GNet.is_unknown_server = False
-        GNet.is_no_meta_data_avail = False
-
     @staticmethod
     def playlist_server(url):
 
@@ -302,18 +281,10 @@ class GNet(GBase):
             sys.exit()
 
         if url[-4:] == '.m3u' or url[-4:] == '.pls':  # or url[-5:] == '.m3u8' or url[-5:] == '.xspf':
-            response = GNet.http_pool.request('GET', url, preload_content=False)
-
-            response_data_split_by_line = response.read().decode("utf-8").splitlines()
+            read_url = GNet.http_pool.request('GET', url, preload_content=False)
+            print(' \n        --- | ---')
+            print(read_url.read().decode('utf-8'), end="")
             print(" \n        --- | ---")
-
-            for _ in response_data_split_by_line:
-                try:
-                    print(_)
-                except UnicodeEncodeError:
-                    pass
-            print(" \n        --- | ---")
-
             try:
                 play_list_server = input('copy and paste a server from above, here ->: ')
             except Exception as ex:
@@ -325,59 +296,22 @@ class GNet(GBase):
             return False
 
         return play_list_server.strip()
-
-    @staticmethod
-    def is_shout_ice_unknown_server(server_port, full_url):
-
-        response = GNet.is_server_alive(server_port)
-        if not response:
-            print(' is_server_alive ' + server_port + 'Can not get the server. Exit')
-            return False
-        else:
-            GNet.is_server_shoutcast(server_port)
-            if GNet.is_shoutcast_server:
-                GNet.is_shoutcast_server = True
-                return GNet.is_shoutcast_server
-            if not GNet.is_shoutcast_server:
-                GNet.is_server_icecast(server_port)
-                if GNet.is_icecast_server:
-                    if GNet.is_server_icecast_status_json(server_port, full_url):
-                        # http://stream.dancewave.online:8080/retrodance.mp3/status-json.xsl full url
-                        GNet.is_icecast_json = True
-                        return GNet.is_icecast_json
-
-            if not GNet.is_icecast_json:
-                GNet.is_unknown_server = True
-                req = GNet.http_pool.request('GET', full_url, headers={'Icy-MetaData': '1'}, preload_content=False)
-                try:
-                    GNet.icy_metadata = int(req.headers['icy-metaint'])
-                    print(f' chunk size {GNet.icy_metadata}')
-                    GIni.cost_dict[GIni.cost_current_ini] = GNet.icy_metadata
-                    # chunk size, calculate request one day (title search)
-                    # write in dict for no metadata = true
-                except Exception as ex:
-                    print(ex)
-                    print(' server has no metadata - try record into one file ')
-                    GNet.is_no_meta_data_avail = True
-                    return GNet.is_no_meta_data_avail
-                return GNet.is_unknown_server
-        return
-
+ 
     @staticmethod
     def is_server_meta_stream(url):
         # print('\n(  ( ((GR)) )  ) \t' + url + ' Click the url to listen.')
         # meta info header, if content-type is found do not go tu suffix check
         try:
             print(url)
-            GNet.request = GNet.http_pool.request('GET', url, headers=GNet.ua_standard, preload_content=False)
+            request = GNet.http_pool.request('GET', url, headers=GNet.ua_standard, preload_content=False)
 
         except Exception as ke:
             print('   --> server failed, no recording ' + url)
             print(ke)
             return False
         else:
-            if GNet.request.status != 200:
-                GNet.request.release_conn()
+            if request.status != 200:
+                request.release_conn()
                 return False
             return True
 
@@ -394,13 +328,13 @@ class GNet(GBase):
             return True
 
     @staticmethod
-    def is_server_shoutcast(server_name):
+    def is_server_shoutcast(server_name, key):
 
         try:
             request = GNet.http_pool.request('GET', server_name, preload_content=False, retries=10)  # winamp
             if not request.headers['icy-notice1'].find('http://www.winamp.com') == -1:  # find() -1 fail
-                GNet.is_shoutcast_server = True
-                GIni.cost_dict[GIni.cost_current_ini] = len(request.headers)
+                GNet.is_shout_dict[key] = True
+                # GIni.cost_dict[GIni.cost_current_ini] = len(request.headers)
         except KeyError:
             # print(repr(ex))
             # print(' no Shoutcast stream')
@@ -409,13 +343,13 @@ class GNet(GBase):
             return True
 
     @staticmethod
-    def is_server_icecast(server_name):
+    def is_server_icecast(server_name, key):
 
         try:
             request = GNet.http_pool.request('GET', server_name, preload_content=False)  # 'server'
             substr_start_num = request.headers['server'].lower().find("icecast")
             if not substr_start_num == -1:  # -1 false from find()
-                GNet.is_icecast_server = True
+                GNet.is_ice_dict[key] = True
                 return True
             else:
                 print(f' server type: {request.headers["server"]}')
@@ -443,23 +377,18 @@ class GNet(GBase):
         return data_stats_source_title
 
     @staticmethod
-    def is_server_icecast_status_json(server_name, full_url):
+    def is_server_icecast_status_json(server_name, full_url, key):
 
         try:
             request = GNet.http_pool.request('GET', server_name + '/status-json.xsl', preload_content=False)
-            # print(f' 1is_server_icecast_status_json ')
             song_title = GNet.ice_status_json(request)
-            # print(f'2is_server_icecast_status_json  {song_title}')
-            GNet.is_icecast_server = True
-            GNet.is_icecast_json = True
 
         except Exception as ex:
             print(ex)
             print(' server got no readable json file \n will read from stream directly \n')
-            # GNet.is_unknown_server = True
             return False
-        else:
-            return song_title
+
+        return song_title
 
     @staticmethod
     def stream_filetype(server_name):
@@ -486,8 +415,40 @@ class GNet(GBase):
 
             return content_type
 
+    @staticmethod
+    def is_shout_ice_unknown_server(server_port, full_url, key):
+
+        response = GNet.is_server_alive(server_port)
+        if not response:
+            print(' is_server_alive ' + server_port + 'Can not get the server. Exit')
+            return
+        else:
+            GNet.is_server_shoutcast(server_port, key)
+            if not GNet.is_shout_dict[key]:
+                GNet.is_server_icecast(server_port, key)
+                if GNet.is_ice_dict[key]:
+                    if GNet.is_server_icecast_status_json(server_port, full_url, key):
+                        # http://stream.dancewave.online:8080/retrodance.mp3/status-json.xsl full url
+                        GNet.is_ice_json_dict[key] = True
+
+            if not GNet.is_ice_json_dict[key]:
+                if not GNet.is_shout_dict[key]:
+                    GNet.is_unk_dict[key] = True
+                    req = GNet.http_pool.request('GET', full_url, headers={'Icy-MetaData': '1'}, preload_content=False)
+                    try:
+                        GNet.icy_metadata = int(req.headers['icy-metaint'])
+                        print(f' chunk size {GNet.icy_metadata} {key}')
+                        # GIni.cost_dict[GIni.cost_current_ini] = GNet.icy_metadata
+                        # chunk size, calculate request one day (title search)
+                        # write in dict for no metadata = true
+                    except Exception as ex:
+                        print(ex)
+                        print(' server has no metadata - try record into one file ')
+                        GNet.is_no_meta_avail_dict[key] = True
+
 
 class GRecorder:
+
     stream_song_name = '_no-name-record_no_split_'
     path_to_song_dict = {}  # each thread writes the new title to the station key name {station : title}
 
@@ -512,7 +473,8 @@ class GRecorder:
             if GIni.srv_param_dict[ini_key + '_is_icecast_json'] == 'True':
                 ice_short_url = GIni.srv_param_dict[url]  # get http://ip:port from full url
                 # ice_short_url = GIni.parse_url_simple_url(url)
-                stream_song_name = GNet.is_server_icecast_status_json(ice_short_url, ice_short_url)
+                # call def for reading json file
+                stream_song_name = GNet.is_server_icecast_status_json(ice_short_url, ice_short_url, ini_key)
 
             if GIni.srv_param_dict[ini_key + '_is_shoutcast_server'] == 'True':
                 srv = GIni.srv_param_dict[url]  # get http://ip:port from full url
@@ -579,6 +541,7 @@ class GRecorder:
         fresh_song = 'False'
         old_time = time()
         sleeper_recorder = False
+
         try:
             if GIni.start_stop_recording[radio_short_key] == 'stop':
                 sleeper_recorder = True
@@ -594,7 +557,7 @@ class GRecorder:
             print('Connection failed. ' + url)
         else:
             # print(' \n Recording ... ')
-            request.auto_close = False  # NOT DEL !!! :)
+            request.auto_close = False  # NOT DEL !!! :) , for continuous recording fine
 
             while not GBase.exit_app:
                 while not GBase.exit_app:
@@ -603,16 +566,17 @@ class GRecorder:
                     else:
                         # print(f'----> else {radio_short_key} sleeper_recorder {sleeper_recorder}'
                         #      f' dict {GIni.start_stop_recording[radio_short_key]}')
-                        # recorder def loose connect if to long idle
                         if sleeper_recorder:
                             print(f'----> start record: {radio_short_key}')
                             try:
+                                # recorder def looses connect if too long idle
                                 request = GNet.http_pool.request('GET', url, preload_content=False, retries=10)
                             except urllib3.exceptions.NewConnectionError:
                                 pass
                         break
                 i = 0
                 while not GBase.exit_app:  # outer loop create empty new files
+
                     if GIni.start_stop_recording[radio_short_key] == 'stop':
                         print(f'----> stop record: {radio_short_key}')
 
@@ -620,10 +584,10 @@ class GRecorder:
 
                     if not fresh_song == stream_song_name:
 
-                        fresh_song = stream_song_name
+                        fresh_song = stream_song_name  # check if it can be removed, we look in a dict now
                         clean_name = GBase.remove_special_chars(stream_song_name)
 
-                        if clean_name == '_no-name-record_no_split_':
+                        if clean_name == '_no-name-record_no_split_':  # init name of title on startup
                             clean_name = clean_name + GBase.this_time()
                             clean_name = GBase.remove_special_chars(clean_name)
 
@@ -632,9 +596,11 @@ class GRecorder:
                             print(f' \t\t\t\t\t\t(  (  ( (Ghetto Recorder) )  )  ) )')
                             i = 0
                         i += 1
+
                         with open(fresh_file, 'wb') as record_file:
 
                             while not GBase.exit_app:  # inner loop get the stream and write into file
+
                                 chunk = request.read(stream_request_size)  # wasting no space in cluster size
                                 record_file.write(chunk)
                                 # this_chunk_len = int(len(chunk))
@@ -650,7 +616,7 @@ class GRecorder:
 
                                 stream_song_name_new = GRecorder.path_to_song_dict[radio_short_key]  # look what's new
 
-                                if not stream_song_name_new == '':
+                                if not stream_song_name_new == '':  # some weired stations send this string
                                     stream_song_name = stream_song_name_new
                                 if not fresh_song == stream_song_name:  # only here break, jump to outer loop
                                     record_file.flush()
@@ -705,6 +671,7 @@ def step1_collect_stream_server():
     except KeyError:
         pass
 
+
     GIni.show_items_ini_file()  # print to terminal
     valid_input = False
     while True:  # collect a list of radio keys to work on
@@ -713,7 +680,7 @@ def step1_collect_stream_server():
             valid_input = True
             # read all keys from ini file
 
-        if ini_file_input == '':  # collect valid keys till enter
+        if ini_file_input == '':  # exit, or valid_input go next
             break
         else:
             valid_input = True
@@ -724,20 +691,16 @@ def step1_collect_stream_server():
                 str_val = GIni.find_ini_file(GBase.remove_special_chars(str_key))
 
                 if str_key == '42':
-                    print('<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< 42 >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>')
                     # if 42, return gives a list instead of a value
                     str_list = GIni.find_all_in_stations()
                     for ini_key in str_list:
                         ini_row_value = str_list[ini_key]
                         add_server_to_data_base(ini_key, ini_row_value)
                 else:
-
                     add_server_to_data_base(str_key, str_val)  # work with one value
 
     if valid_input:
         step2_test_stream_server(GIni.ini_keys)
-    else:
-        print(f' not input - exit')
 
 
 def add_server_to_data_base(str_key, str_val):
@@ -746,6 +709,7 @@ def add_server_to_data_base(str_key, str_val):
     GBase.make_directory(GBase.radio_base_dir + '//' + str_key)
 
     is_playlist_server = GNet.playlist_server(GIni.ini_keys[str_key])
+
     if is_playlist_server:  # update dictionary with new url
         GIni.ini_keys[str_key] = is_playlist_server  # append dictionary, test if it is alive
         if not GNet.is_server_meta_stream(GIni.ini_keys[str_key]):
@@ -758,60 +722,34 @@ def add_server_to_data_base(str_key, str_val):
 
 def step2_test_stream_server(ini_keys):
     # further update dictionary
-    for key in ini_keys:
-        test_stream_server(key)
-    # print(GIni.srv_param_dict)
     # calc_connection_cost()
-    record(ini_keys, GIni.srv_param_dict)  # this loops at end for signal
+    with ThreadPoolExecutor() as executor:
+        futures = {executor.submit(test_stream_server, key) for key in ini_keys}
+
+        for fut in as_completed(futures):
+            pass
+            # print(f"The outcome is {fut.result()} {fut.__dict__}")
+
+    record(ini_keys, GIni.srv_param_dict)  # this starts record threads and loops at end for shutdown signal
 
 
-def test_stream_server(key):
-    ini_keys = GIni.ini_keys
-    GIni.cost_current_ini = key  # for Byte cost calculation
-
-    # write next dictionary with connection parameters
-    GIni.srv_param_dict[key] = ini_keys[key]
-    # print(f'   GIni.srv_param_dict[key]   {GIni.srv_param_dict[key]}    {path_dict[key]}')
-
-    GNet.reset_switch()  # those GNet class vars are modified each loop
-    GIni.srv_param_dict[ini_keys[key]] = GIni.parse_url_simple_url(GIni.srv_param_dict[key])  # http://ip:port
-
-    # str_in is input, modify to get more dictionary keys; file type, is shout is ice ...
-    ret = GNet.stream_filetype(GIni.srv_param_dict[key])
-    GIni.srv_param_dict[key + '_file'] = GNet.stream_filetype(GIni.srv_param_dict[key])
-    if ret is None or ret == '' or ret == ' ':
-        GIni.srv_param_dict[key + '_file'] = '.mp3'
-    # set switch to server type to either read the title from http or from http stream, GNet class vars mod.
-    GNet.is_shout_ice_unknown_server(GIni.srv_param_dict[ini_keys[key]], GIni.srv_param_dict[key])
-    if GNet.is_shoutcast_server:
-        GIni.srv_param_dict[key + '_is_shoutcast_server'] = 'True'
-    else:
-        GIni.srv_param_dict[key + '_is_shoutcast_server'] = 'False'
-    if GNet.is_icecast_server:
-        GIni.srv_param_dict[key + '_is_icecast_server'] = 'True'
-    else:
-        GIni.srv_param_dict[key + '_is_icecast_server'] = 'False'
-    if GNet.is_icecast_json:
-        GIni.srv_param_dict[key + '_is_icecast_json'] = 'True'
-    else:
-        GIni.srv_param_dict[key + '_is_icecast_json'] = 'False'
-    if GNet.is_unknown_server:
-        GIni.srv_param_dict[key + '_is_unknown_server'] = 'True'
-    else:
-        GIni.srv_param_dict[key + '_is_unknown_server'] = 'False'
-    if GNet.is_no_meta_data_avail:
-        GIni.srv_param_dict[key + '_is_no_meta_data_avail'] = 'True'
-    else:
-        GIni.srv_param_dict[key + '_is_no_meta_data_avail'] = 'False'
+def init_dicts_srv_type(key):
 
+    GNet.is_shout_dict[key] = False
+    GNet.is_ice_dict[key] = False
+    GNet.is_ice_json_dict[key] = False
+    GNet.is_unk_dict[key] = False
+    GNet.is_no_meta_avail_dict[key] = False
 
 def calc_connection_cost():
+
     for item in GIni.cost_dict:
         print(f' key: {item} Bytes: {GIni.cost_dict[item]}, title search only - cost per/hour: '
-              f'{round(((int(GIni.cost_dict[item]) * 30 * 60) / 1024) / 1024, 2)}mb')
+              f'{round(((int(GIni.cost_dict[item]) * 30 * 60)/1024) /1024, 2)}mb')
 
 
 def print_stream_server():
+
     print(f'GIni.srv_param_dict {GIni.srv_param_dict}')
 
 
@@ -820,6 +758,7 @@ def record(ini_keys, path_dict_srv_params):
     signal.signal(signal.SIGINT, signal_handler)
 
     for ini_key in ini_keys:
+
         GIni.song_dict[ini_key] = '_no-name-record_no_split_'  # init the dict for this thread
         GIni.start_stop_recording[ini_key] = 'start'
         # GIni.start_stop_recording[ini_key] = 'stop'            # init it here, should be set via user interface
@@ -833,12 +772,61 @@ def record(ini_keys, path_dict_srv_params):
 
         dir_save = GBase.radio_base_dir + '//' + ini_key
         stream_suffix = path_dict_srv_params[ini_key + '_file']
-
+        # print(
+        #     f' url,{url} dir_save:{dir_save} stream_suffix: {stream_suffix} ini_key: {ini_key} '
+        #     f'to_song_dict:{GRecorder.path_to_song_dict}')
         GBase.pool.submit(GRecorder.record_songs, url, dir_save, stream_suffix, ini_key, None)
 
     while not GBase.exit_app:
         sleep(1)
 
+def test_stream_server(key):
+    # srv_param_dict is a simulation of a db for collecting all bool vars, file types, short urls
+    # to fire then the http connects and feed the title pull def with correct connection type: json, http, stream
+    ini_keys = GIni.ini_keys  # for not twisting brain at end of def / the dict with the radio names
+    # GIni.cost_current_ini = key  # for Byte cost calculation / should switch to dict
+
+    # write next dictionary with connection parameters
+    GIni.srv_param_dict[key] = GIni.ini_keys[key]
+    # print(f'   GIni.srv_param_dict[key]   {GIni.srv_param_dict[key]}    {path_dict[key]}')
+
+    GIni.srv_param_dict[ini_keys[key]] = GIni.parse_url_simple_url(GIni.srv_param_dict[key])  # http://ip:port
+
+    # str_in is input, modify to get more dictionary keys; file type, is shout is ice ...
+    ret = GNet.stream_filetype(GIni.srv_param_dict[key])
+
+    GIni.srv_param_dict[key + '_file'] = ret
+    if ret is None or ret == '' or ret == ' ':
+        GIni.srv_param_dict[key + '_file'] = '.mp3'
+
+    # init the dicts with False
+    init_dicts_srv_type(key)
+    # server type, either read the title from http, json file or from http stream
+    # each server type property has a dict, { radio name : False/True}, with a small db it would be much cleaner
+    # GNet.is_shout_ice_unknown_server(server_port, full_url, key)
+    GNet.is_shout_ice_unknown_server(GIni.srv_param_dict[ini_keys[key]], GIni.srv_param_dict[key], key)
+
+    if GNet.is_shout_dict[key]:
+        GIni.srv_param_dict[key + '_is_shoutcast_server'] = 'True'
+    else:
+        GIni.srv_param_dict[key + '_is_shoutcast_server'] = 'False'
+    if GNet.is_ice_dict[key]:
+        GIni.srv_param_dict[key + '_is_icecast_server'] = 'True'
+    else:
+        GIni.srv_param_dict[key + '_is_icecast_server'] = 'False'
+    if GNet.is_ice_json_dict[key]:
+        GIni.srv_param_dict[key + '_is_icecast_json'] = 'True'
+    else:
+        GIni.srv_param_dict[key + '_is_icecast_json'] = 'False'
+    if GNet.is_unk_dict[key]:
+        GIni.srv_param_dict[key + '_is_unknown_server'] = 'True'
+    else:
+        GIni.srv_param_dict[key + '_is_unknown_server'] = 'False'
+    if GNet.is_no_meta_avail_dict[key]:
+        GIni.srv_param_dict[key + '_is_no_meta_data_avail'] = 'True'
+    else:
+        GIni.srv_param_dict[key + '_is_no_meta_data_avail'] = 'False'
+
 
 def main():
     step1_collect_stream_server()
