@@ -1,17 +1,19 @@
+# 0.1.2
 import logging
 from pathlib import Path
 import os
+from tkinter import ttk
+from time import sleep
+from ghettorecorder.lib.edit import UItextEditor
+from ghettorecorder.lib import term
 try:
     import tkinter as tk
 except ImportError:
     print(f'\n\n  Please install --> python3-tk: on Ubuntu sudo apt-get install python3-tk\n\n')
-from tkinter import ttk
-from time import sleep
-from tkinter.filedialog import askdirectory, askopenfilename
-# import UItextEditor
-# import UIghettoTerminal as Gt
-from ghettorecorder import UItextEditor
-from ghettorecorder import UIghettoTerminal as Gt
+try:
+    from tkinter.filedialog import askdirectory, askopenfilename
+except ImportError:
+    print(f'\n error tkinter.filedialog import askdirectory, askopenfilename\n')
 
 # logging.basicConfig(level=logging.DEBUG, format='(%(threadName)-10s) %(message)s',)
 
@@ -19,49 +21,18 @@ from ghettorecorder import UIghettoTerminal as Gt
 class UIWindow(tk.Tk):
 
     box_list = []
-    box_dict = {}  # try dict, list corrupted
     search_list = []
     entry_list = []
     lbl_li = []
-    cb_dic = {}
-
-    box_list_pl = []
-    lbl_li_pl = []
 
     timer_val = ['', 1, 2, 4, 6, 12, 24]
     data_dir_set = False
     record_started = False
     search_started = False
 
-    info_text = [
-        'Die Zeilen in der Mitte sind für Suchbegriffe da.',
-        'Man sollte vorher im linken Kasten den Haken setzen. "Search"',
-        'Begriffe wie: Elvis mozart band Beton, in die Zeile schreiben',
-        'Es ist egal, ob das Wort GROSS oder klein geschrieben ist.',
-        'Das Radio spielt den Titel mit dem Wort, Aufnahme beginnt.',
-        'Bei schnellen Titelwechseln kann noch ein Teil des folgenden',
-        'Titels aufgenommen werden. Einfach löschen.',
-        '"Timer" oben rechts, wenn man nicht selber Stop drücken will.',
-        'Der Timer ist die Eieruhr. nur in Stunden. Fertig! :)',
-        'Die Uhr kann man jederzeit ändern. Viel Spaß!',
-    ]
-
-    info_te_eng = [
-        'The lines in the middle area are used for searching phrases.',
-        'Please activate the corresponding box on the left. "Search"',
-        'Phrases like: Elvis mozart band concrete, can be written.',
-        'There is no rule for writing the words UPPER or lower case.',
-        'Radio station is playing a title with a search phrase! Record.',
-        'Sometimes titles are published before or after the content',
-        'changes. Get rid of the garbage files. Delete it.',
-        'A "Timer" in the upper right corner. If you do not like pressing',
-        'the "Stop" button yourself. One can change the value every time.',
-        'Have fun!'
-    ]
-
     def __init__(self, radio_dir=None, settings_path=None):
         super().__init__()
-        self.gr_version = '0.1.1'
+        self.gr_version = '0.1.3'
         self.radio_dir = radio_dir
         self.settings_path = settings_path
 
@@ -69,10 +40,9 @@ class UIWindow(tk.Tk):
         self.geometry('{}x{}'.format(620, 460))
 
         # main containers create
-        self.top_frame = tk.Frame(master=self, bg='azure3', width=600, height=50, pady=3)
-        self.center = tk.Frame(master=self, bg='gray2', width=50, height=40, padx=3, pady=3)
-        self.info = tk.Frame(master=self, bg='azure3', width=50, height=2, padx=1, pady=1)
-        self.btm_frame = tk.Frame(master=self, bg='azure3', width=600, height=40, pady=3)
+        self.top_frame = tk.Frame(master=self, bg='azure3', width=620, height=50, pady=3)
+        self.center = tk.Frame(master=self, bg='white', width=620, height=360, padx=3, pady=3)
+        self.btm_frame = tk.Frame(master=self, bg='azure3', width=620, height=50, pady=3)
 
         # main containers layout
         self.grid_rowconfigure(1, weight=1)
@@ -80,17 +50,16 @@ class UIWindow(tk.Tk):
 
         self.top_frame.grid(row=0, sticky="ew")
         self.center.grid(row=1, sticky="nsew")
-        self.info.grid(row=2, sticky="nsew")
-        self.btm_frame.grid(row=3, sticky="ew")
+        self.btm_frame.grid(row=2, sticky="ew")
 
         # center widgets create
         self.center.grid_rowconfigure(0, weight=1)
         self.center.grid_columnconfigure(1, weight=1)
 
-        self.ctr_left = tk.Frame(self.center, bg='blue', width=25, height=190)
-        self.canvas = tk.Canvas(self.center, bg='cornflowerblue', width=250, height=190, bd=0)
-        self.ctr_mid = tk.Frame(self.center, bg='azure3', width=250, height=190, padx=3, pady=3)
-        self.ctr_right = tk.Frame(self.center, bg='green', width=25, height=190, padx=3, pady=3)
+        self.ctr_left = tk.Frame(self.center, bg='blue', width=25, height=360)
+        self.canvas = tk.Canvas(self.center, bg='cornflowerblue', width=570, height=360, bd=0)
+        self.ctr_mid = tk.Frame(self.center, bg='azure3', width=570, height=360, padx=3, pady=3)
+        self.ctr_right = tk.Frame(self.center, bg='green', width=25, height=360, padx=3, pady=3)
         # center grid
         self.ctr_left.grid(row=0, column=0, sticky="ns")
         self.canvas.grid(row=0, column=1, sticky="nsew")
@@ -124,13 +93,24 @@ class UIWindow(tk.Tk):
         self.entry_bbr.grid(row=1, column=2, columnspan=3, padx=5, sticky='e')
         self.combo_timer.grid(row=1, column=5, padx=10, sticky='e')
 
+        # Middle
+        self.ghetto_canvas = tk.Canvas(self.ctr_mid, width=600, height=350, bg='azure3')
+        # self.ghetto_canvas = tk.Canvas(self.ctr_mid, width=540, height=350, bg='azure3')
+        self.ghetto_canvas.grid(row=0, column=1, sticky="nsew")
+
+        self.canvas_frame = tk.Frame(self.ghetto_canvas, bg='azure3')
+        self.ghetto_canvas.create_window(0, 0, window=self.canvas_frame, anchor='nw')
+
+        self.canvas_frame.bind('<Enter>', self._bound_to_mousewheel)
+        self.canvas_frame.bind('<Leave>', self._unbound_to_mousewheel)
+
         # center right
         self.progressbar = ttk.Progressbar(self.ctr_right, orient="vertical", length=350, mode="determinate")
         # bottom
 
         self.lbl_rec = tk.Label(master=self.btm_frame, bg='azure3', anchor="s")
         self.btn_editor = tk.Button(master=self.btm_frame, text="Radio Editor",
-                                    width=10,  padx=2, pady=1, command=UItextEditor.UItextEditor)
+                                    width=10,  padx=2, pady=1, command=UItextEditor)
         self.btn_stop = tk.Button(master=self.btm_frame, text="Stop", width=10, padx=2, pady=1, command=self.exit_app)
         self.btn_rec = tk.Button(master=self.btm_frame, text="Record", width=10,
                                  padx=2, pady=1, command=self.list_checked)
@@ -150,7 +130,7 @@ class UIWindow(tk.Tk):
 
         # show path to settings.ini
         ini_file = os.path.join(os.path.dirname(os.path.abspath(__file__)), "settings.ini")
-        Gt.GBase.settings_path = ini_file
+        term.GBase.settings_path = ini_file
         self.entry_ini.delete(0, tk.END)
         self.entry_ini.insert(0, ini_file)
         self.lbl_info.config(text='Oben links der Knopf ist für die Datei mit den Web Radios.')
@@ -161,18 +141,21 @@ class UIWindow(tk.Tk):
     def on_leave(self, enter):
         self.canvas.delete("line")
 
+    def _bound_to_mousewheel(self, event):
+        self.ghetto_canvas.bind_all("<MouseWheel>", self._on_mousewheel)
+
+    def _unbound_to_mousewheel(self, event):
+        self.ghetto_canvas.unbind_all("<MouseWheel>")
+
+    def _on_mousewheel(self, event):
+        self.ghetto_canvas.yview_scroll(int(-1 * (event.delta / 120)), "units")
+
     def set_settings_path(self, settings_path):
         self.settings_path = settings_path
 
-    def get_settings_path(self):
-        return self.settings_path
-
     def set_radio_dir(self, radio_dir):
         self.radio_dir = radio_dir
 
-    def get_radio_dir(self):
-        return self.radio_dir
-
     def browse_dir_button(self):
         dir_name = askdirectory(title='Ghetto Recorder')
         if not dir_name:
@@ -180,7 +163,7 @@ class UIWindow(tk.Tk):
         self.set_radio_dir(dir_name)
         self.entry_bbr.delete(0, tk.END)
         self.entry_bbr.insert(0, dir_name)
-        Gt.GBase.radio_base_dir = dir_name
+        term.GBase.radio_base_dir = dir_name
         # self.lbl_browse.config(text=dir_name)
         self.data_dir_set = True
         return dir_name
@@ -197,10 +180,10 @@ class UIWindow(tk.Tk):
         self.entry_ini.delete(0, tk.END)
         self.entry_ini.insert(0, filepath)
 
-        Gt.GBase.settings_path = filepath  # set new path
-        Gt.GIni.show_items_ini_file()
+        term.GBase.settings_path = filepath  # set new path
+        term.GIni.show_items_ini_file()
         # get the radios
-        self.load_settings(Gt.GIni.list_items, self.ctr_mid)  # only keys, not values
+        self.load_settings(term.GIni.list_items, self.canvas_frame)  # scrollbar
         self.btn_ini = tk.Button(self.top_frame, text="Radio list", width=10, state="disabled")
         self.btn_ini.grid(row=0, column=1, sticky='w')
 
@@ -217,8 +200,8 @@ class UIWindow(tk.Tk):
 
             if not self.data_dir_set:
 
-                Gt.GBase.radio_base_dir = data_folder
-                Gt.GBase.make_directory(data_folder)
+                term.GBase.radio_base_dir = data_folder
+                term.GBase.make_directory(data_folder)
                 self.entry_bbr.delete(0, tk.END)
                 self.entry_bbr.insert(0, data_folder)
 
@@ -228,11 +211,11 @@ class UIWindow(tk.Tk):
             self.entry_bbr.insert(0, 'My directory is read only')
 
     def exit_app(self):
-        Gt.GBase.exit_app = True
-        sleep(Gt.GBase.sleeper + 1)  # threads sleep too, time for exit
+        term.GBase.exit_app = True
+        sleep(term.GBase.sleeper + 1)  # threads sleep too, time for exit
         self.btn_rec = tk.Button(master=self.btm_frame, text="Record", width=10, command=self.list_checked)
         self.btn_rec.grid(row=0, column=3, sticky='w')
-        Gt.GBase.exit_app = False
+        term.GBase.exit_app = False
 
     def cb_checked(self):
         is_playlist_server = ''
@@ -258,13 +241,13 @@ class UIWindow(tk.Tk):
         # remove start screen
         self.ctr_mid.grid()
         self.canvas.grid_remove()
-        tk.Label(frame, text='Record', underline=0 , bg='azure3').grid(row=0, column=2, sticky="e")
-        tk.Label(frame, text='Add search (Record if match - multi): podcast concert elvis', underline=0,
+        tk.Label(frame, text='Record', underline=0, bg='azure3').grid(row=0, column=2, sticky="e")
+        tk.Label(frame, text='Search multiple words (record if match): podcast concert elvis', underline=0,
                  bg='azure3').grid(row=0, column=1, sticky="n")
         tk.Label(frame, text='Search', bg='azure3').grid(row=0, column=0, sticky="w")
         tk.Label(frame, text='Radio', bg='azure3').grid(row=0, column=3, sticky="w")
 
-        Gt.GBase.pool.submit(self.display_info_text)
+        term.GBase.pool.submit(self.display_info_text)
 
         for idx, text in enumerate(list_of_widgets):
 
@@ -284,6 +267,14 @@ class UIWindow(tk.Tk):
             tk.Checkbutton(frame, variable=self.box_list[-1],
                            command=self.cb_checked, bg='azure3').grid(row=idx + 1, column=2, sticky='e')
 
+        # draw the scrollbar
+        ghetto_scroll = tk.Scrollbar(self.ctr_mid, orient=tk.VERTICAL)
+        ghetto_scroll.config(command=self.ghetto_canvas.yview)
+        self.ghetto_canvas.config(yscrollcommand=ghetto_scroll.set)
+        ghetto_scroll.grid(row=0, column=5, sticky="ns")
+
+        self.canvas_frame.bind("<Configure>", self.update_scrollregion)
+
     def list_checked(self):
 
         for ctr, int_var in enumerate(self.box_list):
@@ -291,96 +282,97 @@ class UIWindow(tk.Tk):
             if int_var.get():
 
                 # all get a string, that is never found
-                Gt.GIni.search_dict[self.lbl_li[ctr].cget("text")] = "x!c?42"
+                term.GIni.search_dict[self.lbl_li[ctr].cget("text")] = "x!c?42"
 
                 # SEARCH check only metadata, start rec. if found
                 self.record_started = True
                 if self.search_list[ctr].get():  # not zero = True
                     self.search_started = True
                     # update dict for threads going to search def GRecorder.search_pattern_start_record
-                    Gt.GIni.search_dict[self.lbl_li[ctr].cget("text")] = self.entry_list[ctr].get()
-                    Gt.GIni.search_title_keys_list.append(self.lbl_li[ctr].cget("text"))
+                    term.GIni.search_dict[self.lbl_li[ctr].cget("text")] = self.entry_list[ctr].get()
+                    term.GIni.search_title_keys_list.append(self.lbl_li[ctr].cget("text"))
                     sleep(2)
                     # update label with search content from entry
                     self.lbl_info.config(text=self.entry_list[ctr].get())
 
                 # PLAYLIST is handled in add_server_to..UI, bit diff. than in terminal
                 self.add_server_to_data_base_ui(str(self.lbl_li[ctr].cget("text")),
-                                                str(Gt.GIni.find_ini_file(self.lbl_li[ctr].cget("text"))))
+                                                str(term.GIni.find_ini_file(self.lbl_li[ctr].cget("text"))))
 
         self.btn_rec = tk.Button(master=self.btm_frame, text="Record", width=10, state="disabled")
         self.btn_rec.grid(row=0, column=3, sticky='w')
 
         # here updated list, zombies deleted
-        for key in Gt.GIni.ini_keys:
-            Gt.test_stream_server(key)
+        for key in term.GIni.ini_keys:
+            term.test_stream_server(key)
+        # term.step2_test_stream_server(term.GIni.ini_keys) not yet
 
-        for key in Gt.GIni.ini_keys:
-            Gt.GBase.pool.submit(self.start_records, key)
+        for key in term.GIni.ini_keys:
+            term.GBase.pool.submit(self.start_records, key)
 
-        Gt.GBase.pool.submit(self.display_title)
+        term.GBase.pool.submit(self.display_title)
 
         # if selected, start timer
         if self.combo_timer.get():
             new_timer = int(self.combo_timer.get()) * 3600  # 60s * 60m = ?
-            Gt.GBase.pool.submit(self.timer, new_timer)
+            term.GBase.pool.submit(self.timer, new_timer)
             self.progressbar.grid(row=0, column=0, sticky='s')
 
     @staticmethod
     def start_records(key):
 
-        # ----- no del -- Gt.GRecorder.thread_pull_song_name(url, key, None, None)
-        stream_suffix = Gt.GIni.srv_param_dict[key + '_file']
-        Gt.GIni.song_dict[key] = str('_no-name-record_no_split_')  # init the dict for this thread
-        Gt.GIni.start_stop_recording[key] = 'start'
-        Gt.GIni.start_stop_recording[key + '_adv'] = 'start_from_here'
+        # ----- no del -- term.GRecorder.thread_pull_song_name(url, key, None, None)
+        stream_suffix = term.GIni.srv_param_dict[key + '_file']
+        term.GIni.song_dict[key] = str('_no-name-record_no_split_')  # init the dict for this thread
+        term.GIni.start_stop_recording[key] = 'start'
+        term.GIni.start_stop_recording[key + '_adv'] = 'start_from_here'
 
-        for _ in Gt.GIni.search_title_keys_list:
+        for _ in term.GIni.search_title_keys_list:
             if _ == key:
-                Gt.GIni.start_stop_recording[key] = 'stop'
+                term.GIni.start_stop_recording[key] = 'stop'
 
-        url = Gt.GIni.ini_keys[key]
+        url = term.GIni.ini_keys[key]
         print(f'{key} {url}')
-        dir_save = Gt.GBase.radio_base_dir + '//' + key
-        Gt.GBase.pool.submit(Gt.GRecorder.record_songs, url, dir_save, stream_suffix, key, None)
-        Gt.GRecorder.path_to_song_dict = Gt.GIni.song_dict
-        Gt.GBase.pool.submit(Gt.GRecorder.thread_pull_song_name, url, key, None, None)
+        dir_save = term.GBase.radio_base_dir + '//' + key
+        term.GBase.pool.submit(term.GRecorder.record_songs, url, dir_save, stream_suffix, key, None)
+        term.GRecorder.path_to_song_dict = term.GIni.song_dict
+        term.GBase.pool.submit(term.GRecorder.thread_pull_song_name, url, key, None, None)
 
     def display_title(self):
 
-        while not Gt.GBase.exit_app:
+        while not term.GBase.exit_app:
             for idx, var in enumerate(self.box_list):
                 if var.get():
                     radio = str(self.lbl_li[idx].cget("text"))
                     self.entry_list[idx].delete(0, tk.END)
                     try:
-                        title = Gt.GRecorder.path_to_song_dict[radio]
+                        title = term.GRecorder.path_to_song_dict[radio]
                         self.entry_list[idx].insert(0, title)
                     except Exception as ex:
                         print(ex)
 
             for sec in range(10):
                 sleep(1)
-                if Gt.GBase.exit_app:
+                if term.GBase.exit_app:
                     break
 
     @staticmethod
     def add_server_to_data_base_ui(str_key, str_val):
         is_playlist_server = ''
-        Gt.GIni.ini_keys[str_key] = str_val  # append url to dictionary as value
-        Gt.GBase.make_directory(Gt.GBase.radio_base_dir + '//' + str_key)
+        term.GIni.ini_keys[str_key] = str_val  # append url to dictionary as value
+        term.GBase.make_directory(term.GBase.radio_base_dir + '//' + str_key)
         # playlist url?
         if str_val[-4:] == '.m3u' or str_val[-4:] == '.pls':  # or url[-5:] == '.m3u8' or url[-5:] == '.xspf':
             # take first from the list
             is_playlist_server = UIUtils.playlist_m3u(str_val)
         if not is_playlist_server == '':  # update dictionary with new url
-            Gt.GIni.ini_keys[str_key] = is_playlist_server  # append dictionary, test if it is alive
-            if not Gt.GNet.is_server_meta_stream(Gt.GIni.ini_keys[str_key]):
+            term.GIni.ini_keys[str_key] = is_playlist_server  # append dictionary, test if it is alive
+            if not term.GNet.is_server_meta_stream(term.GIni.ini_keys[str_key]):
                 print('   --> playlist_server server failed, no recording')
-                del Gt.GIni.ini_keys[str_key]
-        if not Gt.GNet.is_server_meta_stream(Gt.GIni.ini_keys[str_key]):  # first time internet access, response code
+                del term.GIni.ini_keys[str_key]
+        if not term.GNet.is_server_meta_stream(term.GIni.ini_keys[str_key]):  # first time internet access, response code
             # delete key from dict, return
-            del Gt.GIni.ini_keys[str_key]
+            del term.GIni.ini_keys[str_key]
 
     def timer(self, time_left):
         combo_time = 0
@@ -393,14 +385,14 @@ class UIWindow(tk.Tk):
                 combo_time = int(self.combo_timer.get()) * 3600
             if not combo_time == time_left:
                 time_left = combo_time
-            if Gt.GBase.exit_app:
+            if term.GBase.exit_app:
                 break
             if not self.record_started:
                 break
 
-        Gt.GBase.exit_app = True
-        sleep(Gt.GBase.sleeper + 1)  # threads sleep too, time for exit
-        Gt.GBase.exit_app = False
+        term.GBase.exit_app = True
+        sleep(term.GBase.sleeper + 1)  # threads sleep too, time for exit
+        term.GBase.exit_app = False
         self.btn_rec = tk.Button(master=self.btm_frame, text="Record", width=10, command=self.list_checked)
         self.btn_rec.grid(row=0, column=3, sticky='w')
 
@@ -415,29 +407,58 @@ class UIWindow(tk.Tk):
 
     def display_info_text(self):
 
-        while not Gt.GBase.exit_app:
+        while not term.GBase.exit_app:
             if self.search_started or self.record_started:
                 self.lbl_info.config(text=' ')
                 break
-            for line in UIWindow.info_text:
+            for line in UIUtils.info_ger:
                 self.lbl_info.config(text=line)
-                for _ in range(10):
+                for _ in range(4):
                     if self.search_started or self.record_started:
                         self.lbl_info.config(text=' ')
                         break
                     sleep(1)
 
-            for line in UIWindow.info_te_eng:
+            for line in UIUtils.info_eng:
                 self.lbl_info.config(text=line)
-                for _ in range(10):
+                for _ in range(4):
                     if self.search_started or self.record_started:
                         self.lbl_info.config(text=' ')
                         break
                     sleep(1)
 
+    def update_scrollregion(self, event):
+        self.ghetto_canvas.configure(scrollregion=self.ghetto_canvas.bbox("all"))
+
 
 class UIUtils(UIWindow):
 
+    info_ger = [
+        'Die Zeilen in der Mitte sind für Suchbegriffe da.',
+        'Dein Lieblingslied im Radio gehört? Suche es.',
+        'Zuerst im linken Kasten den Haken setzen. "Search"',
+        'Worte wie: Elvis Anthrax Heino Beton Kammermusik.',
+        'Es ist egal, ob das Wort GROSS oder klein geschrieben ist.',
+        'Das Radio spielt den Titel mit dem Wort, Aufnahme startet.',
+        'Bei Titelwechseln kann noch ein Teil des folgenden',
+        'Titels aufgenommen werden. Einfach löschen.',
+        '"Timer" oben rechts, keine Zeit Stop zu drücken?',
+        'Der Timer ist die Eieruhr. Nur in Stunden. Fertig! :)',
+        'Die Uhr kann man jederzeit ändern. Viel Spaß!',
+    ]
+
+    info_eng = [
+        'The lines in the middle area are used for searching phrases.',
+        'Your favorite song on air? Search the title or artist.',
+        'Please activate the corresponding box on the left. "Search"',
+        'Phrases like: Elvis Anthrax Heino concrete chamber music.',
+        'There is no rule for writing the words UPPER or lower case.',
+        'Radio station is playing a title with a search phrase! Record!',
+        'A "Timer" in the upper right corner. If you do not like ',
+        'pressing the "Stop" button yourself. One can change the value',
+        ' every time. Have fun!'
+    ]
+
     def __init__(self, radio_dir=None, settings_path=None):
         super().__init__()
 
@@ -445,7 +466,7 @@ class UIUtils(UIWindow):
     def playlist_m3u(url):
         # returns the first server of the playlist
         try:
-            read_url = Gt.GNet.http_pool.request('GET', url, preload_content=False)
+            read_url = term.GNet.http_pool.request('GET', url, preload_content=False)
         except Exception as ex:
             print(ex)
         else:
